'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _jquery = require('jquery');

var _jquery2 = _interopRequireDefault(_jquery);

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function xhrWithProgress(options) {
  return function () {
    var myXhr = _jquery2.default.ajaxSettings.xhr();

    if (myXhr.upload && options.onProgress) {
      myXhr.upload.addEventListener('progress', function (prog) {
        var progress = Math.ceil(prog.loaded / prog.total * 100);
        options.onProgress && options.onProgress(progress || 100);
        // ^ This is just for flow
      }, false);
    }

    return myXhr;
  };
}

function ajaxOptions(options) {
  if (options.method === 'GET') return { data: options.data };

  var formData = new FormData();
  var hasFile = false;

  (0, _lodash.forEach)(options.data, function (val, attr) {
    hasFile = hasFile || val instanceof File;
    if (!(0, _lodash.isNull)(val)) formData.append(attr, val);
  });

  var baseOptions = {
    method: options.method
  };

  if (hasFile) {
    return Object.assign({}, baseOptions, {
      cache: false,
      processData: false,
      data: formData,
      xhr: xhrWithProgress(options),
      contentType: false
    });
  }

  return Object.assign({}, baseOptions, {
    contentType: 'application/json',
    data: options.data ? JSON.stringify(options.data) : null
  });
}

function parseJson(str) {
  try {
    return JSON.parse(str);
  } catch (_error) {
    return null;
  }
}

function ajax(url, options) {
  var xhr = _jquery2.default.ajax(url, ajaxOptions(options));

  var promise = new Promise(function (resolve, reject) {
    xhr.done(resolve).fail(function (jqXHR, _textStatus) {
      var json = parseJson(jqXHR.responseText);
      var ret = json ? json.errors : {};

      return reject(ret || {});
    });
  });

  var abort = function abort() {
    return xhr.abort();
  };

  return { abort: abort, promise: promise };
}

exports.default = {
  apiPath: '',

  get: function get(path, data) {
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return ajax('' + this.apiPath + path, Object.assign({}, { method: 'GET', data: data }, options));
  },
  post: function post(path, data) {
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return ajax('' + this.apiPath + path, Object.assign({}, { method: 'POST', data: data }, options));
  },
  put: function put(path, data) {
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return ajax('' + this.apiPath + path, Object.assign({}, { method: 'PUT', data: data }, options));
  },
  del: function del(path) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    return ajax('' + this.apiPath + path, Object.assign({}, { method: 'DELETE' }, options));
  }
};